# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division

import json

from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.gis.geos import Point, GEOSGeometry
from django.db.models.fields.files import ImageFieldFile
from django.utils.encoding import force_text
from django.utils.functional import Promise


def decorate(*reversed_views):
    """
    provide a syntax decorating views without nested calls.

    instead of:
    json_api_call(etag(<hash_fn>)(<view_fn>)))

    you can write:
    decorate(json_api_call, etag(<hash_fn>), <view_fn>)
    """
    fns = reversed_views[::-1]
    view = fns[0]
    for wrapper in fns[1:]:
        view = wrapper(view)
    return view


# https://docs.djangoproject.com/en/dev/topics/serialization/#id2
class LazyEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        elif hasattr(obj, 'dict'):
            return obj.dict()
        elif isinstance(obj, set):
            return list(obj)
        elif hasattr(obj, 'as_dict'):
            return obj.as_dict()
        elif isinstance(obj, Point):
            srid = 4326
            obj = obj.transform(srid, clone=True)
            return {'x': obj.x, 'y': obj.y, 'srid': srid}
        # TODO: Handle S3
        elif isinstance(obj, ImageFieldFile):
            if obj:
                return obj.url
            else:
                return None
        elif isinstance(obj, GEOSGeometry):
            srid = 4326
            return {
                'geojson': json.loads(obj.transform(srid, clone=True).json),
                'srid': srid
            }
        else:
            return super(LazyEncoder, self).default(obj)



from setuptools import setup, find_packages

author = 'Michael Maurizi'
author_email = 'info@azavea.com'

setup(
    name='django-tinsel',
    version='1.0.2',
    description='A python module for decorating function-based Django views',
    long_description=open('README.rst').read(),
    author=author,
    author_email=author_email,
    maintainer=author,
    maintainer_email=author_email,
    url='http://github.com/azavea/django-tinsel',
    packages=find_packages(exclude=('test_app',)),
    license='Apache License (2.0)',
    keywords="django view decorator",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Environment :: Plugins",
        "Framework :: Django",
        "License :: OSI Approved :: Apache Software License"
    ],
    install_requires=['django>=1.8'],
)
